from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.azure_blob_connection_settings_api_dto import AzureBlobConnectionSettingsApiDto
    from ..models.s3_connection_settings_api_dto import S3ConnectionSettingsApiDto


T = TypeVar("T", bound="CreateConnectionRequest")


@_attrs_define
class CreateConnectionRequest:
    """
    Attributes:
        name (str | Unset):
        description (str | Unset):
        settings (AzureBlobConnectionSettingsApiDto | S3ConnectionSettingsApiDto | Unset):
    """

    name: str | Unset = UNSET
    description: str | Unset = UNSET
    settings: AzureBlobConnectionSettingsApiDto | S3ConnectionSettingsApiDto | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.s3_connection_settings_api_dto import S3ConnectionSettingsApiDto

        name = self.name

        description = self.description

        settings: dict[str, Any] | Unset
        if isinstance(self.settings, Unset):
            settings = UNSET
        elif isinstance(self.settings, S3ConnectionSettingsApiDto):
            settings = self.settings.to_dict()
        else:
            settings = self.settings.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if settings is not UNSET:
            field_dict["settings"] = settings

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.azure_blob_connection_settings_api_dto import AzureBlobConnectionSettingsApiDto
        from ..models.s3_connection_settings_api_dto import S3ConnectionSettingsApiDto

        d = dict(src_dict)
        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        def _parse_settings(data: object) -> AzureBlobConnectionSettingsApiDto | S3ConnectionSettingsApiDto | Unset:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_connection_settings_api_dto_type_0 = S3ConnectionSettingsApiDto.from_dict(data)

                return componentsschemas_connection_settings_api_dto_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_connection_settings_api_dto_type_1 = AzureBlobConnectionSettingsApiDto.from_dict(data)

            return componentsschemas_connection_settings_api_dto_type_1

        settings = _parse_settings(d.pop("settings", UNSET))

        create_connection_request = cls(
            name=name,
            description=description,
            settings=settings,
        )

        create_connection_request.additional_properties = d
        return create_connection_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.query_workflow_request_inputs_type_0_item import QueryWorkflowRequestInputsType0Item


T = TypeVar("T", bound="QueryWorkflowRequest")


@_attrs_define
class QueryWorkflowRequest:
    """
    Attributes:
        inputs (list[QueryWorkflowRequestInputsType0Item] | None | Unset):
    """

    inputs: list[QueryWorkflowRequestInputsType0Item] | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        inputs: list[dict[str, Any]] | None | Unset
        if isinstance(self.inputs, Unset):
            inputs = UNSET
        elif isinstance(self.inputs, list):
            inputs = []
            for inputs_type_0_item_data in self.inputs:
                inputs_type_0_item = inputs_type_0_item_data.to_dict()
                inputs.append(inputs_type_0_item)

        else:
            inputs = self.inputs

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if inputs is not UNSET:
            field_dict["inputs"] = inputs

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.query_workflow_request_inputs_type_0_item import QueryWorkflowRequestInputsType0Item

        d = dict(src_dict)

        def _parse_inputs(data: object) -> list[QueryWorkflowRequestInputsType0Item] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                inputs_type_0 = []
                _inputs_type_0 = data
                for inputs_type_0_item_data in _inputs_type_0:
                    inputs_type_0_item = QueryWorkflowRequestInputsType0Item.from_dict(inputs_type_0_item_data)

                    inputs_type_0.append(inputs_type_0_item)

                return inputs_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[QueryWorkflowRequestInputsType0Item] | None | Unset, data)

        inputs = _parse_inputs(d.pop("inputs", UNSET))

        query_workflow_request = cls(
            inputs=inputs,
        )

        query_workflow_request.additional_properties = d
        return query_workflow_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.orchestrator_status import OrchestratorStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workflow_status import WorkflowStatus


T = TypeVar("T", bound="WorkflowStatusResponse")


@_attrs_define
class WorkflowStatusResponse:
    """
    Attributes:
        workflow_rid (str | Unset):
        orchestrator_status (OrchestratorStatus | Unset):
        workflow_status (None | Unset | WorkflowStatus):
    """

    workflow_rid: str | Unset = UNSET
    orchestrator_status: OrchestratorStatus | Unset = UNSET
    workflow_status: None | Unset | WorkflowStatus = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.workflow_status import WorkflowStatus

        workflow_rid = self.workflow_rid

        orchestrator_status: str | Unset = UNSET
        if not isinstance(self.orchestrator_status, Unset):
            orchestrator_status = self.orchestrator_status.value

        workflow_status: dict[str, Any] | None | Unset
        if isinstance(self.workflow_status, Unset):
            workflow_status = UNSET
        elif isinstance(self.workflow_status, WorkflowStatus):
            workflow_status = self.workflow_status.to_dict()
        else:
            workflow_status = self.workflow_status

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if workflow_rid is not UNSET:
            field_dict["workflowRid"] = workflow_rid
        if orchestrator_status is not UNSET:
            field_dict["orchestratorStatus"] = orchestrator_status
        if workflow_status is not UNSET:
            field_dict["workflowStatus"] = workflow_status

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.workflow_status import WorkflowStatus

        d = dict(src_dict)
        workflow_rid = d.pop("workflowRid", UNSET)

        _orchestrator_status = d.pop("orchestratorStatus", UNSET)
        orchestrator_status: OrchestratorStatus | Unset
        if isinstance(_orchestrator_status, Unset):
            orchestrator_status = UNSET
        else:
            orchestrator_status = OrchestratorStatus(_orchestrator_status)

        def _parse_workflow_status(data: object) -> None | Unset | WorkflowStatus:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                workflow_status_type_0 = WorkflowStatus.from_dict(data)

                return workflow_status_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | WorkflowStatus, data)

        workflow_status = _parse_workflow_status(d.pop("workflowStatus", UNSET))

        workflow_status_response = cls(
            workflow_rid=workflow_rid,
            orchestrator_status=orchestrator_status,
            workflow_status=workflow_status,
        )

        workflow_status_response.additional_properties = d
        return workflow_status_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class OrchestratorStatus(str, Enum):
    CANCELED = "CANCELED"
    COMPLETED = "COMPLETED"
    CONTINUED_AS_NEW = "CONTINUED_AS_NEW"
    FAILED = "FAILED"
    RUNNING = "RUNNING"
    TERMINATED = "TERMINATED"
    TIMED_OUT = "TIMED_OUT"
    UNSPECIFIED = "UNSPECIFIED"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkerRunEventRatingSecondaryId")


@_attrs_define
class WorkerRunEventRatingSecondaryId:
    """
    Attributes:
        run_rid (str):
        sequence (int | Unset):
    """

    run_rid: str
    sequence: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        run_rid = self.run_rid

        sequence = self.sequence

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "runRid": run_rid,
            }
        )
        if sequence is not UNSET:
            field_dict["sequence"] = sequence

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        run_rid = d.pop("runRid")

        sequence = d.pop("sequence", UNSET)

        worker_run_event_rating_secondary_id = cls(
            run_rid=run_rid,
            sequence=sequence,
        )

        worker_run_event_rating_secondary_id.additional_properties = d
        return worker_run_event_rating_secondary_id

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.list_all_file_systems_response_file_systems import ListAllFileSystemsResponseFileSystems


T = TypeVar("T", bound="ListAllFileSystemsResponse")


@_attrs_define
class ListAllFileSystemsResponse:
    """
    Attributes:
        file_systems (ListAllFileSystemsResponseFileSystems | Unset):
    """

    file_systems: ListAllFileSystemsResponseFileSystems | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        file_systems: dict[str, Any] | Unset = UNSET
        if not isinstance(self.file_systems, Unset):
            file_systems = self.file_systems.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if file_systems is not UNSET:
            field_dict["fileSystems"] = file_systems

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.list_all_file_systems_response_file_systems import ListAllFileSystemsResponseFileSystems

        d = dict(src_dict)
        _file_systems = d.pop("fileSystems", UNSET)
        file_systems: ListAllFileSystemsResponseFileSystems | Unset
        if isinstance(_file_systems, Unset):
            file_systems = UNSET
        else:
            file_systems = ListAllFileSystemsResponseFileSystems.from_dict(_file_systems)

        list_all_file_systems_response = cls(
            file_systems=file_systems,
        )

        list_all_file_systems_response.additional_properties = d
        return list_all_file_systems_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.resource_metadata_dto import ResourceMetadataDto


T = TypeVar("T", bound="ResourceEntryDto")


@_attrs_define
class ResourceEntryDto:
    """
    Attributes:
        resource_id (str | Unset):
        metadata (ResourceMetadataDto | Unset):
    """

    resource_id: str | Unset = UNSET
    metadata: ResourceMetadataDto | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        resource_id = self.resource_id

        metadata: dict[str, Any] | Unset = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if resource_id is not UNSET:
            field_dict["resourceId"] = resource_id
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.resource_metadata_dto import ResourceMetadataDto

        d = dict(src_dict)
        resource_id = d.pop("resourceId", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: ResourceMetadataDto | Unset
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = ResourceMetadataDto.from_dict(_metadata)

        resource_entry_dto = cls(
            resource_id=resource_id,
            metadata=metadata,
        )

        resource_entry_dto.additional_properties = d
        return resource_entry_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.indexing_action_status import IndexingActionStatus
from ...models.list_indexing_actions_response import ListIndexingActionsResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    task_rid: str,
    *,
    limit: int | Unset = 100,
    offset: int | Unset = 0,
    status: list[IndexingActionStatus] | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    params["limit"] = limit

    params["offset"] = offset

    json_status: list[str] | Unset = UNSET
    if not isinstance(status, Unset):
        json_status = []
        for status_item_data in status:
            status_item = status_item_data.value
            json_status.append(status_item)

    params["status"] = json_status

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/v1/indexing/tasks/{task_rid}/actions".format(
            task_rid=quote(str(task_rid), safe=""),
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> ListIndexingActionsResponse | None:
    if response.status_code == 200:
        response_200 = ListIndexingActionsResponse.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[ListIndexingActionsResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    task_rid: str,
    *,
    client: AuthenticatedClient | Client,
    limit: int | Unset = 100,
    offset: int | Unset = 0,
    status: list[IndexingActionStatus] | Unset = UNSET,
) -> Response[ListIndexingActionsResponse]:
    """List Indexing Actions

    Args:
        task_rid (str):
        limit (int | Unset):  Default: 100.
        offset (int | Unset):  Default: 0.
        status (list[IndexingActionStatus] | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ListIndexingActionsResponse]
    """

    kwargs = _get_kwargs(
        task_rid=task_rid,
        limit=limit,
        offset=offset,
        status=status,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    task_rid: str,
    *,
    client: AuthenticatedClient | Client,
    limit: int | Unset = 100,
    offset: int | Unset = 0,
    status: list[IndexingActionStatus] | Unset = UNSET,
) -> ListIndexingActionsResponse | None:
    """List Indexing Actions

    Args:
        task_rid (str):
        limit (int | Unset):  Default: 100.
        offset (int | Unset):  Default: 0.
        status (list[IndexingActionStatus] | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ListIndexingActionsResponse
    """

    return sync_detailed(
        task_rid=task_rid,
        client=client,
        limit=limit,
        offset=offset,
        status=status,
    ).parsed


async def asyncio_detailed(
    task_rid: str,
    *,
    client: AuthenticatedClient | Client,
    limit: int | Unset = 100,
    offset: int | Unset = 0,
    status: list[IndexingActionStatus] | Unset = UNSET,
) -> Response[ListIndexingActionsResponse]:
    """List Indexing Actions

    Args:
        task_rid (str):
        limit (int | Unset):  Default: 100.
        offset (int | Unset):  Default: 0.
        status (list[IndexingActionStatus] | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ListIndexingActionsResponse]
    """

    kwargs = _get_kwargs(
        task_rid=task_rid,
        limit=limit,
        offset=offset,
        status=status,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    task_rid: str,
    *,
    client: AuthenticatedClient | Client,
    limit: int | Unset = 100,
    offset: int | Unset = 0,
    status: list[IndexingActionStatus] | Unset = UNSET,
) -> ListIndexingActionsResponse | None:
    """List Indexing Actions

    Args:
        task_rid (str):
        limit (int | Unset):  Default: 100.
        offset (int | Unset):  Default: 0.
        status (list[IndexingActionStatus] | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ListIndexingActionsResponse
    """

    return (
        await asyncio_detailed(
            task_rid=task_rid,
            client=client,
            limit=limit,
            offset=offset,
            status=status,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ManagedFileSystemApiDto")


@_attrs_define
class ManagedFileSystemApiDto:
    """
    Attributes:
        rid (str | Unset):
        name (str | Unset):
        description (str | Unset):
        type_ (str | Unset):
    """

    rid: str | Unset = UNSET
    name: str | Unset = UNSET
    description: str | Unset = UNSET
    type_: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        rid = self.rid

        name = self.name

        description = self.description

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if rid is not UNSET:
            field_dict["rid"] = rid
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        rid = d.pop("rid", UNSET)

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        type_ = d.pop("type", UNSET)

        managed_file_system_api_dto = cls(
            rid=rid,
            name=name,
            description=description,
            type_=type_,
        )

        managed_file_system_api_dto.additional_properties = d
        return managed_file_system_api_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
